# ----------------------------------------------------------------------
# Copyright (C) 2004,2005 Mark Lawrence <nomad@null.net>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# ----------------------------------------------------------------------
# Read from rotating log files
# ----------------------------------------------------------------------
package Log::Read;
use 5.006;
use strict;
use warnings;
use Carp;
use POSIX qw(tmpnam);
use Time::localtime;
use IO::File;
use debug;

our $VERSION = '0.02';


sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self  = {};
    bless ($self, $class);

    my %param = (
        file        => '',
        statefile   => '',
        debug       => 0,
        @_
    );

    if ($param{file} eq '' or $param{statefile} eq '') {
        croak 'usage: ', __PACKAGE__,
              '->new(file => $file, statefile => $file)';
    }

    $self->{debug} = $param{debug};

    #
    # Get a file handle for our log file
    #
    $self->{file} = $param{file};
    $self->{fh}   = $self->IOFile($self->{file});

    if (!defined($self->{fh})) {
        return undef;
    }

    $self->{end}  = ($self->{fh}->stat)[7];


    #
    # Find out the last position we reached in the file
    #
    $self->{statefile} = $param{statefile};
    if (-r $self->{statefile}) {
        if (! -w $self->{statefile}) {
            warn "Not able to write to $self->{statefile}";
            return undef;
        }
        my $fh = IO::File->new("< $self->{statefile}");
        if (!$fh) {
            warn "Failed to open $self->{statefile}: $!";
            return undef;
        }
        my $tmp = '';
        while (my $line = $fh->getline()) {
            chomp($line);
            if ($line =~ /^pos: ([0-9]+)$/) {
                $self->{'pos'} = $1;
            }
            if ($line =~ /^file: (.*)$/) {
                $tmp = $1;
            }
            if ($line =~ /^data: (.*)$/) {
                $self->{data} = $1;
            }
        }
        if (!defined($self->{'pos'})) {
            warn "$self->{statefile} is not a valid statefile\n";
            if (!open(TMP, '>',$self->{statefile})) {
                warn "Could not open $self->{statefile}: $!\n";
                return undef;
            }
            close(TMP);
            $self->{'pos'} = 0;
            $self->{data}  = '';
        }
        #
        # If this is not the same file as the last look then reset the
        # position counter.
        #
        if ($tmp ne $self->{file}) {
            warn 'Filename does not match statefile contents';
        }
    }
    else {
        if (!open(TMP, '>',$self->{statefile})) {
            warn "Could not open $self->{statefile}: $!\n";
            return undef;
        }
        close(TMP);
        $self->{'pos'} = 0;
        $self->{data}  = '';
    }

    debug::log("Initial State: File=$self->{file} Pos=$self->{pos} End=$self->{end}") if(DEBUG);


    #
    # Check for rotation
    #
    if ($self->{pos} > $self->{end}) {
        $self->get_rot_fh();
    }
    else { 
        $self->{fh}->seek($self->{pos}, 0);
    }

    $self->{lines} = 0;
    return $self;
}

#
# Get the File Handle of the Rotated file
#
sub get_rot_fh {
    my $self = shift;

    my $now       = time();
    my $today     = localtime($now);
    my $yesterday = localtime($now - 60*60*24);

    $today = sprintf("%4u%02u%02u",
                     $today->year + 1900,
                     $today->mon + 1,
                     $today->mday
    );

    $yesterday = sprintf("%4u%02u%02u",
                         $yesterday->year + 1900,
                         $yesterday->mon + 1,
                         $yesterday->mday
    );

    my $rotlogfile = undef;

    for my $ext ('.0',
                 '.0.gz',
                 '.1',
                 '.1.gz',
                 '.01.gz',
                 "-$today",
                 "-$today.gz",
                 "-$yesterday",
                 "-$yesterday.gz") {

        if (-f "$self->{file}$ext") {
            $self->{rotatedfile} = $self->{file} . $ext;
            $self->{rfh}         = $self->IOFile($self->{rotatedfile});
            if (!defined($self->{rfh})) {
                delete $self->{rotatedfile};
                delete $self->{rfh};
                return;
            }
            $self->{rfh}->seek($self->{'pos'}, 0);
            $self->{end} = ($self->{rfh}->stat)[7];
            debug::log("Rotated: File=$self->{rotatedfile} Pos=$self->{pos} End=$self->{end}") if(DEBUG);
            return;
        }
    }

    warn "Could not find rotated version of $self->{file}\n";
    return;
}


sub getline {
    my $self = shift;
    if (defined $self->{rfh}) {
        if ($self->{'pos'} < $self->{end}) {
            my $line = $self->{rfh}->getline;
            $self->{'pos'} = $self->{rfh}->tell;
            $self->{lines}++;
            return $line;
        }
        elsif ($self->{'pos'} == $self->{end}) {
            delete $self->{rfh};
            $self->{end}   = ($self->{fh}->stat)[7];
            $self->{'pos'} = 0;
            debug::log("File=$self->{file} Pos=$self->{pos} End=$self->{end}") if(DEBUG);
        }
        else {
            carp "Internal error: pos($self->{pos}) is greater than end($self->{end})\n";
            return undef;
        }
    }
    if ($self->{'pos'} < $self->{end}) {
        my $line = $self->{fh}->getline;
        $self->{'pos'} = $self->{fh}->tell;
        $self->{lines}++;
        return $line;
    }
    return undef;
}

#
# IO::File but handles gzipped files (in read mode only). Should replace
# this method with IO::Zip when 'tell' and 'seek' functionality is
# implemented in that module.
#
# Usage: IOFile($file) - returns an IO::File object or undef upon failure
#
sub IOFile {
    my $self  = shift;
    my $fname = shift;
    my $fh;

    if (!open(TMPF, $fname)) {
        warn "Could not open $fname: $!\n";
        return undef;
    }
    close(TMPF);

    #
    # Is this a gzipped file?
    #
    if ($fname =~ /\.gz$/i) {
        #
        # Try new temporary filenames until we get one that didn't
        # already exist
        #
        my $tmpname;
        do {
            $tmpname = tmpnam()
        } until $fh = IO::File->new($tmpname, O_RDWR|O_CREAT|O_EXCL);

        #
        # Save the file name for later
        #
        push(@{$self->{tmpfiles}}, $tmpname);

        my $system = "gzip -dc $fname > $tmpname";
        if (system($system) != 0) {
            warn "system: '$system' failed: $!";
            return undef;
        }
        $fname = $tmpname;
    }

    #
    # Now open and return the real file
    #
    $fh = IO::File->new("< $fname");
    if (!$fh) {
        warn "Failed to open file $fname: $!\n";
        return undef;
    }
    return $fh;
}

#
# User-specific data - gets saved to the statefile when we finish.
#
sub data {
    my $self = shift;
    $self->{data} = shift if (@_);
    return $self->{data};
}


sub save_state {
    my $self = shift;
    return unless(defined($self->{'pos'}));

    debug::log("Saving State: File=$self->{file} Pos=$self->{pos}. Lines: $self->{lines}") if(DEBUG);

    my $fh = IO::File->new("> $self->{statefile}");

    print $fh 'pos: '  . $self->{'pos'} . "\n";
    print $fh 'file: ' . $self->{file} . "\n";
    print $fh 'data: ' . $self->{data} . "\n";
}

sub DESTROY {
    my $self = shift;
    $self->save_state();
    if ($self->{tmpfiles}) {
        unlink(@{$self->{tmpfiles}}) or carp "Could not delete tmpfiles: $!";
    }
}


1;

__END__

=head1 NAME

Log::Read - read from rotating log files

=head1 SYNOPSIS

  use Log::Read;
  my $fh = new Log::Read(file => $file, statefile => $statefile);
  die unless($fh);

  while (my $line = $fh->getline) {
      # do something with $line
  }

  $fh->save_state();

=head1 DESCRIPTION

B<Log::Read> allows you to easily read regularly from files that might
be rotated and compressed - for example syslog(8) files - without worrying
about finding the rotated log file, decompressing it and seeking to 
where you previously read from.

This module is mostly useful for logfile analysis programs run at regular
intervals from cron(8).

=head1 METHODS

=head2 new( file => $file, statefile => $statefile)

Create a "Log::Read" object representing $file with the state
stored in $statefile.

If $statefile does not exist it will be created and the file pointer
will be set to zero (start of $file).

If $statefile is valid for $file the file pointer will be moved to the
last read position (in either $file or its rotated version).

=head2 $fh->getline( )

Will return the next string from $file or the rotated version of
$file. When the end of the rotated file is reached lines will be returned
from the current $file. When the last line is reached returns 'undef'.

=head2 $fh->save_state( )

Save the current file state to $statefile. This should be called
at regular intervals or when you want to record the position you reached
in the file.

=head2 $fh->data( ),   $fh->data( $data )

This method allows you to set and read your own string in the $statefile.
Your string cannot contain any newlines (all data after a newline is lost).
This might be useful for keeping track of counters inbetween invocations.

=head1 SEE ALSO

L<IO::File>

=head1 AUTHOR

Mark Lawrence E<lt>nomad@null.netE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2004,2005 Mark Lawrence <nomad@null.net>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

=cut

