# ----------------------------------------------------------------------
# Copyright (C) 2004,2005 Mark Lawrence <nomad@null.net>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# ----------------------------------------------------------------------
# Read from rotating log files
# ----------------------------------------------------------------------
package Log::Read::Exim;

use 5.006;
use strict;
use warnings;
use Carp;
use Log::Read;
use DateTime;
use debug;

our @ISA = qw(Log::Read);
our $VERSION = '0.01';


sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;

    my $self = __PACKAGE__->SUPER::new(@_);
    return undef if (!$self);

    bless ($self, $class);

    $self->{date} = DateTime->now();

    return $self;
}


sub geteline {
    my $self = shift;

    my $line;
    while ($line = $self->getline()) {
        next unless ($line =~ s/^([0-9]{4})-([0-9]{2})-([0-9]{2})\s+([0-9]{2}):([0-9]{2}):([0-9]{2})\s+//o);

        my ($year,$mon,$day,$hour,$min,$sec) = ($1,$2,$3,$4,$5,$6);

        eval {
            $self->{date}->set(
                second => $sec,
                minute => $min,
                hour   => $hour,
                day    => $day,
                month  => $mon,
                year   => $year,
            );
        };
        next if ($@);

        return ($self->{date}->epoch(), $line);
    }
    return undef;
}


1;

__END__


=head1 NAME

Log::Read::Exim - read from rotating exim files

=head1 SYNOPSIS

  use Log::Read::Exim;
  my $fh = new Log::Read::Exim(file      => $file,
                                  statefile => $statefile);
  die unless($fh);

  while ((my ($epoch, $line) = $fh->geteline) == 2) {
      last unless($epoch);
      # do something with $epoch and $line
  }

  $fh->save_state();

=head1 DESCRIPTION

B<Log::Read::Exim> allows you to easily read regularly from exim
log files that might be rotated and compressed, without worrying
about finding the rotated log file, decompressing it and seeking to 
where you previously read from.

This module is mostly useful for logfile analysis programs run at regular
intervals from cron(8). Derives directly from Log::Read and provides
one extra function dealing specifically with the exim(8) line format:

  2005-01-26 14:08:13 1Ctmtf-0000kK-1M => noone@lksjdflks.com.au R=smarthost...

Requires DateTime from CPAN.

=head1 METHODS

=head2 $fh->geteline( )

Returns a two item list derived from the next line in the file. The first item
is the date ('2005-01-26 14:08:13') converted into the number of seconds since
1 Jan 1970 (1101702670). The second item is a string containing the content
of the log entry ('1Ctmtf-0000kK-1M => noone@lksjdflks.com.au R=smarthost..').

When the end of the rotated file is reached lines will be returned
from the current $file. When the last line is reached returns 'undef'.

=head2 Log::Read methods

L<Log::Read::Exim> derives from Log::Read and so all methods
from that class are available as well.

=head1 SEE ALSO

L<Log::Read>, L<DateTime>

=head1 AUTHOR

Mark Lawrence E<lt>nomad@null.netE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2004,2005 Mark Lawrence <nomad@null.net>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

=cut


