# ----------------------------------------------------------------------
# Copyright (C) 2004,2005 Mark Lawrence <nomad@null.net>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# ----------------------------------------------------------------------
# Read from rotating syslog files
# ----------------------------------------------------------------------
package Log::Read::Syslog;

use 5.006;
use strict;
use warnings;
use Carp;
use POSIX qw(strftime);
use POSIX qw(locale_h);
use Log::Read;
use DateTime;
use debug;

our @ISA = qw(Log::Read);
our $VERSION = '0.01';

my $locale = setlocale(LC_TIME);
#print STDERR "locale is set to \"$locale\"\n";

my %months;
for (0..11) {
    $months{strftime("%b", 0, 0, 0, 1, $_, 96)} = $_ + 1;
}


sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;

    my $self = __PACKAGE__->SUPER::new(@_);
    return undef if (!$self);

    bless ($self, $class);

    $self->{date} = DateTime->now(time_zone => 'local');
    return $self;
}


sub geteline {
    my $self = shift;
    my %param = (
        prog => '.*?',
        @_
    );

    my $now = time();
    my $line;
    while ($line = $self->getline()) {
        next unless ($line =~ s/^(.*?)\s+([0-9]{1,2})\s+([0-9]{2}):([0-9]{2}):([0-9]{2})\s+//);

        my ($mon,$day,$hour,$min,$sec) = ($1,$2,$3,$4,$5);

        eval {
            $self->{date}->set(
                second   => $sec,
                minute   => $min,
                hour     => $hour,
                day      => $day,
                month    => $months{$mon},
            );
        };
        next if ($@);

        if ($self->{date}->epoch() > ($now + 43200)) { # 60*60*12 = 1 day
            $self->{date}->subtract(years => 1);
        }

        return ($self->{date}->epoch(), $line);
    }
    return undef;
}


sub getesline {
    my $self = shift;
    my %param = (
        prog => '.*?',
        @_
    );

    my $now = time();
    my $line;
    while ($line = $self->getline()) {
        next unless ($line =~ s/^(.*?)\s+([0-9]{1,2})\s+([0-9]{2}):([0-9]{2}):([0-9]{2})\s+(.*?)\s+$param{prog}\[[0-9]+\]:\s*//);

        my ($mon,$day,$hour,$min,$sec,$host) = ($1,$2,$3,$4,$5,$6);

        eval {
            $self->{date}->set(
                second   => $sec,
                minute   => $min,
                hour     => $hour,
                day      => $day,
                month    => $months{$mon},
            );
        };
        next if ($@);

        if ($self->{date}->epoch() > ($now + 43200)) { # 60*60*12 = 1 day
            $self->{date}->subtract(years => 1);
        }

        return ($self->{date}->epoch(), $line);
    }
    return undef;
}


1;
__END__

=head1 NAME

Log::Read::Syslog - read from rotating syslog files

=head1 SYNOPSIS

  use Log::Read::Syslog;
  my $fh = new Log::Read::Syslog(file      => $file,
                                 statefile => $statefile);
  die unless($fh);

  while ((my ($epoch, $line) = $fh->geteline) == 2) {
      # do something with $epoch and $line
  }

  $fh->save_state();

=head1 DESCRIPTION

B<Log::Read::Syslog> allows you to easily read regularly from syslog
files that might be rotated and compressed, without worrying
about finding the rotated log file, decompressing it and seeking to 
where you previously read from.

This module is mostly useful for logfile analysis programs run at regular
intervals from cron(8). Derives directly from Log::Read and provides
one extra function dealing specifically with the syslog(8) date format:

  Nov 29 04:31:10 mx01us spamd[25064]: connection from localhost

Requires DateTime from CPAN.

=head1 METHODS

=head2 $fh->geteline()

Returns a two item list derived from the next line in the file. The first item
is the date ('Nov 29 04:31:10') converted into the number of seconds since
1 Jan 1970 (1101702670). The second item is the rest of the line (mx01...).

When the end of the rotated file is reached lines will be returned
from the current $file. When the last line is reached returns 'undef'.

=head2 Log::Read methods

L<Log::Read::Syslog> derives from Log::Read and so all methods
from that class are available as well.

=head1 SEE ALSO

L<Log::Read>, L<DateTime>

=head1 AUTHOR

Mark Lawrence E<lt>nomad@null.netE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2004,2005 Mark Lawrence <nomad@null.net>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

=cut

